﻿using System;
using System.Collections.Generic;
using System.Linq;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Banky
{
	// *************************************************************************
	// *** PohybySuborSlsp *****************************************************
	// *************************************************************************
	public class PohybySuborSlsp : PohybySubor
	{
		// ---------------------------------------------------------------------
		private const int IndexStlpcaDatumSpracovania = 0;
		private const int IndexStlpcaFinancnaMena = 8;
		private const int IndexStlpcaRozsirenyPopisInkaso = 6;
		private const int IndexStlpcaCiastka = 7;
		private const int IndexStlpcaTypObratu = 11;
		private const int IndexStlpcaRozsirenyPopisPlatba1 = 16;
		private const int IndexStlpcaRozsirenyPopisPlatba2 = 18;
		private const int IndexStlpcaRozsirenyPoznamka = 19;
		private const int IndexStlpcaRozsirenyPopisTP = 22;
		private const int IndexStlpcaDatumTransakcie = 22;

		// ---------------------------------------------------------------------
		// --- PohybySubor Metody ----------------------------------------------
		// ---------------------------------------------------------------------
		private static DruhObratu DajDruhObratu(string popis)
		{
			if (!string.IsNullOrEmpty(popis))
			{
				popis = popis.ToLower();

				// Ma prednost pred vyberom (pr. "Bezhotovosnty vyber trvaly prik./inkaso")
				bool jeTPInkaso =
					popis.Contains("trvalý príkaz") ||
					popis.Contains("inkaso") ||
					popis == "debetné transakcie-úvery";
				if (jeTPInkaso)
					return DruhObratu.TPaInkaso;

				bool jeVyber = popis.Contains("výber");
				if (jeVyber)
					return DruhObratu.Vyber;

				// Ma prednost pred platbou
				bool jePoplatok =
					popis.Contains("poplatok") ||
					popis.Contains("popl.");	// pr "cezhranicny popl."
				if (jePoplatok)
					return DruhObratu.Poplatok;

				bool jePlatba =
					popis.Contains("platba") ||
					popis.Contains("prevod") ||
					popis.Contains("príkaz cez telebanking") ||
					popis.Contains("príkaz na úhradu");
				if (jePlatba)
					return DruhObratu.Platba;

				bool jeUrok = popis.Contains("úrok");
				if (jeUrok)
					return DruhObratu.Urok;

				bool jeDan = popis.Contains("daň");
				if (jeDan)
					return DruhObratu.Dan;

				bool jeVklad = popis.Contains("vklad");
				if (jeVklad)
					return DruhObratu.Vklad;

				bool jeIrelevantny = popis.Contains("zruš.trvalého");
				if (jeIrelevantny)
					return DruhObratu.Irelevantne;
			} // if

			throw new NotSupportedException("Neznámy druh obratu: " + popis);
		}

		// ---------------------------------------------------------------------
		private static PenaznaMena DajPenaznuMenu(string hodnota)
		{
			if (!string.IsNullOrEmpty(hodnota))
			{
				switch (hodnota.ToLower())
				{
					case "eur":
						return PenaznaMena.Euro;
					
					case "skk":
						return PenaznaMena.Skk;
				} // switch
			} // if

			throw new NotSupportedException("Neznámy peňažná mena: " + hodnota);
		}

		// ---------------------------------------------------------------------
		private static bool CezInternet(string popis)
		{
			if (string.IsNullOrEmpty(popis))
				return false;

			popis = popis.ToLower();
			return popis.Contains("telebanking");
		}

		// ---------------------------------------------------------------------
		private static int DajIndexStlpcaDatumu(DruhObratu druhObratu, bool jeElektronicky)
		{
			// Pre rozne druhy obratu treba pouzit rozne datumy
			switch (druhObratu)
			{
				case DruhObratu.Vklad:
				case DruhObratu.TPaInkaso:
				case DruhObratu.Poplatok:
				case DruhObratu.Urok:
				case DruhObratu.Dan:
					return IndexStlpcaDatumSpracovania;

				default:
					return jeElektronicky ? IndexStlpcaDatumSpracovania : IndexStlpcaDatumTransakcie;
			} // switch
		}

		// ---------------------------------------------------------------------
		protected virtual int[] DajIndexStlpcaRozsrirenaPoznamka(DruhObratu obrat, bool jeElektronicky)
		{
			var zoznam = new List<int>();
			
			zoznam.Add(IndexStlpcaRozsirenyPoznamka);

			if (obrat == DruhObratu.Platba)
			{
				if (jeElektronicky)
					zoznam.Add(IndexStlpcaRozsirenyPopisTP);

				zoznam.Add(IndexStlpcaRozsirenyPopisPlatba1);	// 25.10.2011;000000;0446507818;000000;0000000000;0900;;-77.99;EUR;;2825.64;Platba kartou;0000000000;0000000000;0000000000;0;B143329245-44406 26** **** 4278,SHELL - KE, JUZNE NABREZI ,KOSICE ,SK ,23.10.2011 12:18:07  ,CashBack:0 EUR;0;;;;;;;;
				zoznam.Add(IndexStlpcaRozsirenyPopisPlatba2);
			} // if
			else if (obrat == DruhObratu.TPaInkaso)
			{
				zoznam.Add(IndexStlpcaRozsirenyPopisInkaso);
				zoznam.Add(IndexStlpcaRozsirenyPopisTP);
			} // else if
			else
				zoznam.Add(IndexStlpcaRozsirenyPopisPlatba2);

			return zoznam.ToArray();
		}

		// ---------------------------------------------------------------------
		public override Obrat ParsujVetu(string veta)
		{
			string[] udaje = veta.Split(';');

			if (udaje == null || udaje.Length < 23)
				throw new NotSupportedException("Neznámy formát záznamu!");

			// Nacitame data
			DruhObratu druhObratu = DajDruhObratu(udaje[IndexStlpcaTypObratu]);
			
			// Pokusime sa nacitat datum, ak sa aj tak nenasiel, tak zoberieme prvy (bankovy)
			bool jeElektronicky = CezInternet(udaje[IndexStlpcaTypObratu]);
			string hodnotaDatumu = udaje[DajIndexStlpcaDatumu(druhObratu, jeElektronicky)];
			hodnotaDatumu = string.IsNullOrEmpty(hodnotaDatumu) || !hodnotaDatumu.IsDateTime() ? udaje[IndexStlpcaDatumSpracovania] : hodnotaDatumu;

			var poznamky = new List<string>(DajIndexStlpcaRozsrirenaPoznamka(druhObratu, jeElektronicky).Select(item => udaje[item]));
			string rozsirenaPoznamka = string.Join(", ", poznamky.Where(item => !string.IsNullOrEmpty(item)).ToArray());

			return new Obrat(
				/*identifikator*/veta.ToMd5(),
				/*datum*/OverHodnotu<DateTime>(hodnotaDatumu, "Dátum"),
				/*poznamka*/udaje[IndexStlpcaTypObratu] + (string.IsNullOrEmpty(rozsirenaPoznamka) ? string.Empty : " (" + rozsirenaPoznamka.Trim() + ")"),
				/*druhObratu*/druhObratu,
				/*ciastka*/Math.Abs(OverHodnotu<double>(udaje[IndexStlpcaCiastka], "Čiastka")),
				/*mena*/DajPenaznuMenu(udaje[IndexStlpcaFinancnaMena]));
		}
	} // public class PohybySuborSlsp

} // namespace EvidenciaUctov.SLSP