﻿using System;
using System.Data.SQLite;

namespace EvidenciaUctov.DB
{
	// *************************************************************************
	// *** DataReader **********************************************************
	// *************************************************************************
	public class DataReader : object, IDisposable
	{
		// ---------------------------------------------------------------------
		// --- DataReader Constructor ------------------------------------------
		// ---------------------------------------------------------------------
		public DataReader(SQLiteDataReader sqliteDataReader)
		{
			Reader = sqliteDataReader;
		}

		// ---------------------------------------------------------------------
		// --- DataReader Properties -------------------------------------------
		// ---------------------------------------------------------------------
		public SQLiteDataReader Reader { get; private set; }

		// ---------------------------------------------------------------------
		// --- DataReader Methods ----------------------------------------------
		// ---------------------------------------------------------------------
		public bool Read()
		{
			return Reader.Read();
		}

		// ---------------------------------------------------------------------
		public void Close()
		{
			Reader.Close();
		}

		// ---------------------------------------------------------------------
		public bool IsDBNull(string fieldName)
		{
			return Reader.IsDBNull(Reader.GetOrdinal(fieldName));
		}

		// ---------------------------------------------------------------------
		public string ReadString(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? null : Reader.GetString(ordinal);
		}

		// ---------------------------------------------------------------------
		public char? ReadChar(string fieldName)
		{
			string value = ReadString(fieldName);
			return value == string.Empty ? (char?)null : value[0];
		}

		// ---------------------------------------------------------------------
		public byte? ReadByte(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? (byte?)null : Reader.GetByte(ordinal);
		}

		// ---------------------------------------------------------------------
		public short? ReadInt16(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? (short?)null : Reader.GetInt16(ordinal);
		}

		// ---------------------------------------------------------------------
		public int? ReadInt32(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? (int?)null : Reader.GetInt32(ordinal);
		}

		// ---------------------------------------------------------------------
		public long? ReadInt64(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? (long?)null : Reader.GetInt64(ordinal);
		}

		// ---------------------------------------------------------------------
		internal float? ReadFloat(int ordinal)
		{
			return Reader.IsDBNull(ordinal) ? (float?)null : Reader.GetFloat(ordinal);
		}

		// ---------------------------------------------------------------------
		public float? ReadFloat(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return ReadFloat(ordinal);
		}

		// ---------------------------------------------------------------------
		public double? ReadDouble(string fieldName, double? defaultValue = null)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? defaultValue : Reader.GetDouble(ordinal);
		}

		// ---------------------------------------------------------------------
		public DateTime? ReadDateTime(string fieldName)
		{
			int ordinal = Reader.GetOrdinal(fieldName);
			return Reader.IsDBNull(ordinal) ? (DateTime?)null : Reader.GetDateTime(ordinal);
		}

		// ---------------------------------------------------------------------
		public bool? ReadBoolean(string fieldName, bool? defaultValue = null)
		{
			int? intValue = ReadInt32(fieldName);
			return !intValue.HasValue ? defaultValue : (intValue.Value == 1);
		}

		// ---------------------------------------------------------------------
		public object GetValue(string fieldName)
		{
			return Reader[fieldName];
		}

		// ---------------------------------------------------------------------
		public object GetValue(int fieldIndex)
		{
			return Reader[fieldIndex];
		}

		// ---------------------------------------------------------------------
		public object GetValue(string fieldName, Type type)
		{
			if (type.Equals(typeof(float?)) || type.Equals(typeof(float)))
				return ReadFloat(fieldName);

			return GetValue(fieldName);
		}

		// ---------------------------------------------------------------------
		#region IDisposable

		// ---------------------------------------------------------------------
		public void Dispose()
		{
			if (Reader != null)
				Reader.Dispose();
		}

		#endregion // IDisposable
	} // public class DataReader

} // namespace Elmhurst.Utils.Database.SQLite