﻿using System;
using System.Data;

namespace EvidenciaUctov.DB
{
	// *************************************************************************
	// *** Utils ***************************************************************
	// *************************************************************************
	/// <summary>
	/// Utilsy pre pracu s SQLite databazami
	/// </summary>
	internal static class Utils
	{
		// ---------------------------------------------------------------------
		// --- Utils Metody ----------------------------------------------------
		// ---------------------------------------------------------------------
		public static string DateToSQLString(DateTime date)
		{
			if (date == DateTime.MinValue)
				return "NULL";

			return "DATE('" + date.ToString("yyyy-MM-dd") + "')";
		}

		// ---------------------------------------------------------------------
		public static Context VytvoritContext(string subor, string heslo)
		{
			return ContextFactory.Create(subor, heslo);
		}

		// ---------------------------------------------------------------------
		public static Context VytvoritContextPrihlaseny(Context parentContext)
		{
			return VytvoritContextPrihlaseny(/*commandText*/null, parentContext);
		}

		// ---------------------------------------------------------------------
		public static Context VytvoritContextPrihlaseny(string commandText = null, Context parentContext = null)
		{
			Context context;
			if (parentContext == null)
			{
				if (!Layer.JePrihlaseny())
					throw new UnauthorizedAccessException("Užívateľ musí byť pripojený!");

				context = ContextFactory.Create(Layer.Databaza, Layer.Heslo);
			} // if
			else
				context = ContextFactory.Create(parentContext);

			// Nastavime context
			if (!string.IsNullOrEmpty(commandText))
			{
				context.Command.CommandText = commandText;
				context.Command.Parameters.Clear();
			} // if

			return context;
		}

		// ---------------------------------------------------------------------
		public static int GetLastInsertID(string tableName, Context parentContext = null)
		{
			using (var context = VytvoritContextPrihlaseny("SELECT last_insert_rowid() AS Identity FROM " + tableName, parentContext))
			{
				using (var reader = new DataReader(context.Command.ExecuteReader(CommandBehavior.SingleResult)))
				{
					if (!reader.Read())
						throw new Exception("Interná chyba: Tabuľka '" + tableName + "' neobsahuje žiaden záznam");

					return reader.ReadInt32("Identity").Value;
				} // using
			} // using
		}
	} // public static class Utils

} // namespace EvidenciaUctov.DB