﻿using System.Linq;
using System.Windows;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** FilterDialog ********************************************************
	// *************************************************************************
	public partial class FilterDialog : Window
	{
		// ---------------------------------------------------------------------
		// --- FilterDialog Konstruktor ----------------------------------------
		// ---------------------------------------------------------------------
		public FilterDialog()
		{
			InitializeComponent();

			this.Inicializovat(TextBoxPopis);
			NacitatSkupinyFiltrov();
		}

		// ---------------------------------------------------------------------
		// --- VytvoritDbDialog Udalosti ---------------------------------------
		// ---------------------------------------------------------------------
		private void ButtonPridatSkupinu_Click(object sender, RoutedEventArgs e)
		{
			new SkupinyFitrovDialog().ShowDialog();
			NacitatSkupinyFiltrov();
		}

		// ---------------------------------------------------------------------
		private void ButtonOK_Click(object sender, RoutedEventArgs e)
		{
			DialogResult = true;
		}

		// ---------------------------------------------------------------------
		// --- FilterDialog Metody ---------------------------------------------
		// ---------------------------------------------------------------------
		protected void Nacitat(MesacnaStatistikaFilter mesacnaStatistikaFilter)
		{
			if (mesacnaStatistikaFilter == null)
				return;

			TextBoxPopis.Text = mesacnaStatistikaFilter.Popis;
			ComboBoxSkupina.Oznacit<SkupinaFiltrovMesacnejStatistiky>(item => item.SkupinaFiltrovID == mesacnaStatistikaFilter.SkupinaFiltrovID);
		}

		// ---------------------------------------------------------------------
		protected MesacnaStatistikaFilter Zapisat()
		{
			return new MesacnaStatistikaFilter()
			{
				Popis = TextBoxPopis.Text,
				SkupinaFiltrovID = (ComboBoxSkupina.SelectedValue == null) ? null : (ComboBoxSkupina.SelectedValue as SkupinaFiltrovMesacnejStatistiky).SkupinaFiltrovID
			};
		}

		// ---------------------------------------------------------------------
		protected void NacitatSkupinyFiltrov()
		{
			int? poslednaSkupinaFiltrovID = null;
			if (ComboBoxSkupina.SelectedValue != null)
				poslednaSkupinaFiltrovID = (ComboBoxSkupina.SelectedValue as SkupinaFiltrovMesacnejStatistiky).SkupinaFiltrovID;
			
			ComboBoxSkupina.ItemsSource = DB.Layer.NacitatSkupinyFiltrov().OrderBy(item => item.Popis);

			if (poslednaSkupinaFiltrovID.HasValue && ComboBoxSkupina.SelectedValue == null)
				ComboBoxSkupina.Oznacit<SkupinaFiltrovMesacnejStatistiky>(item => item.SkupinaFiltrovID == poslednaSkupinaFiltrovID);
		}

		// ---------------------------------------------------------------------
		#region Zobrazovace

		// ---------------------------------------------------------------------
		public static int? NovyZaznam()
		{
			return EditovatZaznam(/*filterID*/null);
		}

		// ---------------------------------------------------------------------
		public static int? EditovatZaznam(int? filterID)
		{
			var dialog = new FilterDialog();

			if (filterID.HasValue)
				dialog.Nacitat(DB.Layer.FilterNacitat(filterID.Value));

			if (dialog.ShowDialog() ?? false)
			{
				App.Uzamknut();
				try // finally
				{
					// Vyskladame data z dialogu
					var data = dialog.Zapisat();
					data.FilterID = filterID;

					if (!string.IsNullOrEmpty(data.Popis))
					{
						// Ulozime
						DB.Layer.FilterUlozit(data);
						
						return data.FilterID;
					} // if
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if

			return null;
		}

		#endregion // Zobrazovace
	} // public partial class FilterDialog

} // namespace EvidenciaUctov.Dialogy