﻿using System.Windows;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** FilterPodmienkaDialog ***********************************************
	// *************************************************************************
	public partial class FilterPodmienkaDialog : Window
	{
		// ---------------------------------------------------------------------
		private readonly int filterID;
		private readonly int? filterPodmienkaID;

		// ---------------------------------------------------------------------
		// --- FilterPodmienkaDialog Konstruktor -------------------------------
		// ---------------------------------------------------------------------
		public FilterPodmienkaDialog(int? filterPodmienkaID, int filterID)
		{
			InitializeComponent();

			this.Inicializovat(TextBoxFormat);
			this.filterID = filterID;
			this.filterPodmienkaID = filterPodmienkaID;
			ComboBoxDruhObratu.ItemsSource = EnumUtils.GetEnumDictionaryList(/*vynechat*/DruhObratu.Irelevantne);
		}

		// ---------------------------------------------------------------------
		// --- FilterPodmienkaDialog Vlastnosti --------------------------------
		// ---------------------------------------------------------------------
		#region Mapovanie

		// ---------------------------------------------------------------------
		public DruhObratu ZvolenyDruhObratu
		{
			get { return ComboBoxDruhObratu.SelectedIndex < 0 ? DruhObratu.Neznamy : ComboBoxDruhObratu.SelectedValue.GetEnumValue<DruhObratu>(); } // get
			set { ComboBoxDruhObratu.SetEnumDictionaryListValue(value); } // set
		}

		#endregion // Mapovanie

		// ---------------------------------------------------------------------
		// --- FilterPodmienkaDialog Udalosti ----------------------------------
		// ---------------------------------------------------------------------
		private void ButtonOK_Click(object sender, RoutedEventArgs e)
		{
			DialogResult = true;
		}
	
		// ---------------------------------------------------------------------
		// --- FilterPodmienkaDialog Metody ------------------------------------
		// ---------------------------------------------------------------------
		protected MesacnaStatistikaFilterPodmienka Zapisat()
		{
			return new MesacnaStatistikaFilterPodmienka(TextBoxFormat.Text, filterID, ZvolenyDruhObratu) { FilterPodmienkaID = filterPodmienkaID };
		}

		// ---------------------------------------------------------------------
		protected void Nacitat(MesacnaStatistikaFilterPodmienka data)
		{
			if (data == null)
				return;

			TextBoxFormat.Text = data.Format;
			ZvolenyDruhObratu = data.DruhObratu;
		}

		// ---------------------------------------------------------------------
		#region Zobrazenia

		// ---------------------------------------------------------------------
		public static int? NovyZaznam(int filterID)
		{
			return EditovatZaznam(/*data*/null, filterID);
		}

		// ---------------------------------------------------------------------
		public static int? EditovatZaznam(MesacnaStatistikaFilterPodmienka data, int filterID)
		{
			int? filterPodmienkaID = (data != null ? data.FilterPodmienkaID : null);
			var dialog = new FilterPodmienkaDialog(filterPodmienkaID, filterID);
			dialog.Nacitat(data);

			if (dialog.ShowDialog() ?? false)
			{
				App.Uzamknut();
				try // finally
				{
					// Nacitame hodnoty z dialogu
					data = dialog.Zapisat();

					if (!string.IsNullOrEmpty(data.Format))
					{
						// Vyskladame data a ulozime
						DB.Layer.FilterPodmienkaUlozit(data);

						return data.FilterID;
					} // if
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if

			return null;
		}

		#endregion // Zobrazenia
	} // public partial class FilterPodmienkaDialog

} // namespace EvidenciaUctov.Dialogy