﻿using System;
using System.Collections.Generic;
using System.Windows;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** FilterPridat ********************************************************
	// *************************************************************************
	public partial class FilterPridat : Window
	{
		// ---------------------------------------------------------------------
		private readonly List<PolozkaFiltra> Filtre = new List<PolozkaFiltra>()
		{
			new FilterCislo("Čiastka", "Ciastka"),
			new FilterRetazec("Popis", "Poznamka")
		};

		// ---------------------------------------------------------------------
		// --- FilterPridat Konstruktor ----------------------------------------
		// ---------------------------------------------------------------------
		public FilterPridat()
		{
			InitializeComponent();
			this.Inicializovat();

			ComboBoxFilter.ItemsSource = Filtre;
		}

		// ---------------------------------------------------------------------
		// --- FilterPridat Vlastnosti -----------------------------------------
		// ---------------------------------------------------------------------
		public PolozkaFiltra Filter
		{
			get
			{
				var filter = ComboBoxFilter.SelectedValue as PolozkaFiltra;

				if (filter != null)
				{
					filter.Operator = OperatorHodnot;
					filter.Hodnota = TextBoxHodnota.Text;

					if (filter.JeValidny())
						return filter;
				} // if

				throw new Exception("Chybne nakonfigurovaný filter!");
			} // get
		}

		// ---------------------------------------------------------------------
		#region Mapovanie

		// ---------------------------------------------------------------------
		public OperatorHodnot OperatorHodnot
		{
			get { return ComboBoxOperator.SelectedIndex < 0 ? OperatorHodnot.Ziaden : ComboBoxOperator.SelectedValue.GetEnumValue<OperatorHodnot>(); } // get
			set { ComboBoxOperator.SetEnumDictionaryListValue(value); } // set
		}

		#endregion // Mapovanie

		// ---------------------------------------------------------------------
		// --- FilterPridat Udalosti -------------------------------------------
		// ---------------------------------------------------------------------
		private void ComboBoxFilter_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			var filter = ComboBoxFilter.SelectedValue as PolozkaFiltra;

			ComboBoxOperator.ItemsSource = (filter == null) ? null : EnumUtils.GetEnumDictionaryListFromArray(filter.DajZoznamPodporovanychOperatorov());
		}

		// ---------------------------------------------------------------------
		private void ButtonOK_Click(object sender, RoutedEventArgs e)
		{
			DialogResult = true;
		}
	} // public partial class FilterPridat

} // namespace EvidenciaUctov.Dialogy