﻿using System.IO;
using System.Windows;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** PrihlasovaciDialog **************************************************
	// *************************************************************************
	public partial class PrihlasovaciDialog : Window
	{
		// ---------------------------------------------------------------------
		// --- PrihlasovaciDialog Konstruktor ----------------------------------
		// ---------------------------------------------------------------------
		public PrihlasovaciDialog()
		{		
			InitializeComponent();			

			TextBoxDatabaza.Text = Properties.Settings.Default.PoslednaDatabaza;
			this.Inicializovat(string.IsNullOrEmpty(TextBoxDatabaza.Text) ? (UIElement)TextBoxDatabaza : (UIElement)PasswordBox);
		}

		// ---------------------------------------------------------------------
		// --- PrihlasovaciDialog Udalosti -------------------------------------
		// ---------------------------------------------------------------------
		private void ButtonDatabaza_Click(object sender, RoutedEventArgs e)
		{
			Microsoft.Win32.OpenFileDialog dialog = new Microsoft.Win32.OpenFileDialog()
			{
				Filter = "DB|*.db3",
				DefaultExt = "db3"
			};

			if (!string.IsNullOrEmpty(TextBoxDatabaza.Text))
				dialog.InitialDirectory = Path.GetFullPath(TextBoxDatabaza.Text);

			if (dialog.ShowDialog() ?? false)
				TextBoxDatabaza.Text = dialog.FileName;
		}

		// ---------------------------------------------------------------------
		private void ButtonVytvoritDB_Click(object sender, RoutedEventArgs e)
		{
			VytvoritDbDialog dialog = new VytvoritDbDialog();
			if (dialog.ShowDialog() ?? false)
			{
				Microsoft.Win32.SaveFileDialog dialogSubor = new Microsoft.Win32.SaveFileDialog()
				{
					Filter = "DB|*.db3",
					DefaultExt = "db3"
				};

				if (!string.IsNullOrEmpty(TextBoxDatabaza.Text))
					dialogSubor.InitialDirectory = Path.GetFullPath(TextBoxDatabaza.Text);

				if (dialogSubor.ShowDialog() ?? false)
					DB.Layer.VytvoritDatabazu(dialogSubor.FileName, dialog.PasswordBoxHeslo.Password);

				Prihlasit(dialogSubor.FileName, PasswordBox.Password);
			} // if
		}

		// ---------------------------------------------------------------------
		private void ButtonPrihasit_Click(object sender, RoutedEventArgs e)
		{
			try // catch
			{
				Prihlasit(TextBoxDatabaza.Text, PasswordBox.Password);
			} // try
			catch
			{
				Dispatcher.Focus(PasswordBox);
				throw;
			} // atch
		}

		// ---------------------------------------------------------------------
		// --- PrihlasovaciDialog Metody ---------------------------------------
		// ---------------------------------------------------------------------
		private void Prihlasit(string databaza, string heslo)
		{
			// Rovno uzivatela prihlasime (pripadne updatneme DB)
			DB.Layer.Prihlasit(App.VerziaDB, databaza, heslo);
			Properties.Settings.Default.PoslednaDatabaza = databaza;
			DialogResult = true;
		}
	} // public partial class PrihlasovaciDialog

} // namespace EvidenciaUctov.Dialogy