﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** StatistikaMesiac ****************************************************
	// *************************************************************************
	public partial class StatistikaMesiac : Window
	{
		// ---------------------------------------------------------------------
		private int ucetID;

		// ---------------------------------------------------------------------
		// --- StatistikaMesiac Konstruktor ------------------------------------
		// ---------------------------------------------------------------------
		public StatistikaMesiac(int ucetID)
		{		
			InitializeComponent();

			this.Inicializovat(TextBoxRok);
			this.ucetID = ucetID;
		}

		// ---------------------------------------------------------------------
		// --- StatistikaMesiac Udalosti ---------------------------------------
		// ---------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			TextBoxRok.Text = (App.Current.MainWindow as MainWindow).FilterControlMain.TextBoxRok.Text;
			
			NacitatStatistiky();
		}

		// ---------------------------------------------------------------------
		private void ButtonObnovit_Click(object sender, RoutedEventArgs e)
		{
			NacitatStatistiky();
		}

		// ---------------------------------------------------------------------
		private void ButtonPrisposobit_Click(object sender, RoutedEventArgs e)
		{
			new StatistikaMesiacPrisposobit().ShowDialog();
			NacitatStatistiky();
		}

		// ---------------------------------------------------------------------
		private void DockPanel_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			if (e.Key == Key.F5)
				NacitatStatistiky();
		}

		// ---------------------------------------------------------------------
		// --- StatistikaMesiac Metody -----------------------------------------
		// ---------------------------------------------------------------------
		private void NacitatStatistiky()
		{
			App.Uzamknut();
			try // finally
			{
				TextBoxLog.Clear();

				// Rok
				if (!string.IsNullOrEmpty(TextBoxRok.Text) && !TextBoxRok.Text.IsInt())
				{
					Dispatcher.Focus(TextBoxRok);
					throw new Exception("Chybný rok!");
				} // if

				// Jednotlive riadky statistiky
				PridatStatistiku(/*mesiace*/string.Empty, "Jan", "Feb", "Marec", "Apríl", "Máj", "Jún", "Júl", "Aug", "Sep", "Okt", "Nov", "Dec", "Priemer", "Spolu");
				PridatStatistiku("Výdavky spolu:", DB.Layer.MesacnaStatistikaVydavky(ucetID, TextBoxRok.Text.ToInt(), /*ignorovane*/false));
				PridatStatistiku("+ Ignorované:", DB.Layer.MesacnaStatistikaVydavky(ucetID, TextBoxRok.Text.ToInt(), /*ignorovane*/true));
				
				// Skupinove statistiky
				var skupiny = DB.Layer.NacitatSkupinyFiltrov();
				if (skupiny.Count > 0)
				{
					TextBoxLog.Text += "\nSkupiny:\n";
					foreach (SkupinaFiltrovMesacnejStatistiky skupina in skupiny)
					{
						var filtreVSkupine = DB.Layer.NacitatFiltre().Where(item => item.SkupinaFiltrovID == skupina.SkupinaFiltrovID);						
						if (filtreVSkupine.Count() > 0)
						{
							List<MesacnaStatistikaFilterPodmienka> vsetkyPodmienkyFiltrov = new List<MesacnaStatistikaFilterPodmienka>();
							foreach (var aktualnePodmienky in DajPodmienkyFiltra(filtreVSkupine).Values)
								vsetkyPodmienkyFiltrov.AddRange(aktualnePodmienky);

							PridatStatistiku(skupina.Popis + ":", DB.Layer.MesacnaStatistikaUzivatelska(ucetID, TextBoxRok.Text.ToInt(), vsetkyPodmienkyFiltrov));
						} // if
					} // foreach
				} // if

				// Uzivatelske statistiky
				var podmienky = DajUzivatelskePodmienky();
				if (podmienky.Count > 0)
				{
					TextBoxLog.Text += "\nDetaily:\n";
					foreach (string nazov in podmienky.Keys)
						PridatStatistiku(nazov + ":", DB.Layer.MesacnaStatistikaUzivatelska(ucetID, TextBoxRok.Text.ToInt(), podmienky[nazov]));
				} // if
			} // try
			finally
			{
				App.Odomknut();
			} // finally
		}

		// ---------------------------------------------------------------------
		private void PridatStatistiku(string text, List<double> ciastky)
		{
			// Pridame agregacne stlpce
			double priemer = ciastky.Average();
			double spolu = ciastky.Sum();
			ciastky.Add(priemer);
			ciastky.Add(spolu);

			PridatStatistiku(text, (from ciastka in ciastky select ciastka.ToString("C2")).ToArray());
		}

		// ---------------------------------------------------------------------
		private void PridatStatistiku(string text, params string[] ciastky)
		{
			string riadok = text.PadRight(15);
			foreach (string ciastka in ciastky)
				riadok += ciastka.PadLeft(StatistikaZakladDialog.ZarovnanieCiastka);

			TextBoxLog.Text += riadok + Environment.NewLine;
		}

		// ---------------------------------------------------------------------
		private Dictionary<string, List<MesacnaStatistikaFilterPodmienka>> DajUzivatelskePodmienky()
		{
			return DajPodmienkyFiltra(DB.Layer.NacitatFiltre());
		}

		// ---------------------------------------------------------------------
		private Dictionary<string, List<MesacnaStatistikaFilterPodmienka>> DajPodmienkyFiltra(IEnumerable<MesacnaStatistikaFilter> filtre)
		{
			Dictionary<string, List<MesacnaStatistikaFilterPodmienka>> result = new Dictionary<string, List<MesacnaStatistikaFilterPodmienka>>();

			foreach (MesacnaStatistikaFilter filter in filtre.Where(item => !item.Ignorovat))
			{
				List<MesacnaStatistikaFilterPodmienka> podmienky = DB.Layer.NacitatPodmienkyFiltra(filter.FilterID.Value);
				if (podmienky != null && podmienky.Count > 0)
					result.Add(filter.Popis, podmienky);
			} // foreach

			return result;
		}
	} // public partial class StatistikaMesiac

} // namespace EvidenciaUctov.Dialogy