﻿using System.Linq;
using System.Windows;
using System.Windows.Input;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** StatistikaMesiacPrisposobit *****************************************
	// *************************************************************************
	public partial class StatistikaMesiacPrisposobit : Window
	{
		// ---------------------------------------------------------------------
		// --- StatistikaMesiacPrisposobit Konstruktor -------------------------
		// ---------------------------------------------------------------------
		public StatistikaMesiacPrisposobit()
		{
			InitializeComponent();
		
			this.Inicializovat();
			InicializovatFiltre();
		}

		// ---------------------------------------------------------------------
		// --- StatistikaMesiacPrisposobit Udalosti ----------------------------
		// ---------------------------------------------------------------------
		#region Filter

		// ---------------------------------------------------------------------
		private void ListBoxFilter_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			switch (e.Key)
			{
				case Key.Delete:
					VymazatFilter();
					break;

				case Key.F8:
					IgnorovatFilter();
					break;

				case Key.Enter:
					EditovatFilter();
					break;
			} // switch
		}

		// ---------------------------------------------------------------------
		private void ListBoxFilter_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			InicializovatPodmienkyFiltra();
		}

		// ---------------------------------------------------------------------
		private void ListBoxFilter_MouseDoubleClick(object sender, MouseButtonEventArgs e)
		{
			EditovatFilter();
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilter_ClickPridat(object sender, RoutedEventArgs e)
		{
			int? oznacitNovyFilterID = FilterDialog.NovyZaznam();
			if (oznacitNovyFilterID.HasValue)
				InicializovatFiltre(oznacitNovyFilterID);
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilter_ClickEditovat(object sender, RoutedEventArgs e)
		{
			EditovatFilter();
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilter_ClickIgnorovat(object sender, RoutedEventArgs e)
		{
			IgnorovatFilter();
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilter_ClickVymazat(object sender, RoutedEventArgs e)
		{
			VymazatFilter();
		}

		#endregion // Filter

		// ---------------------------------------------------------------------
		#region Podmienka Filtra

		// ---------------------------------------------------------------------
		private void ListBoxFilterPodmienka_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			switch (e.Key)
			{
				case Key.Delete:
					VymazatFilterPodmienka();
					break;

				case Key.Enter:
					EditovatFilterPodmienka();
					break;
			} // switch
		}

		// ---------------------------------------------------------------------
		private void ListBoxFilterPodmienka_MouseDoubleClick(object sender, MouseButtonEventArgs e)
		{
			EditovatFilterPodmienka();
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilterPodmienka_ClickPridat(object sender, RoutedEventArgs e)
		{
			if (ListBoxFilter.SelectedValue == null)
				return;

			int? oznacitNovuPodmienkuFiltraID = FilterPodmienkaDialog.NovyZaznam(((MesacnaStatistikaFilter)ListBoxFilter.SelectedValue).FilterID.Value);
			if (oznacitNovuPodmienkuFiltraID.HasValue)
				InicializovatPodmienkyFiltra(oznacitNovuPodmienkuFiltraID);
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilterPodmienka_ClickEditovat(object sender, RoutedEventArgs e)
		{
			EditovatFilterPodmienka();
		}

		// ---------------------------------------------------------------------
		private void ToolbarFilterPodmienka_ClickVymazat(object sender, RoutedEventArgs e)
		{
			VymazatFilterPodmienka();
		}

		#endregion // Podmienka

		// ---------------------------------------------------------------------
		// --- StatistikaMesiacPrisposobit Metody ------------------------------
		// ---------------------------------------------------------------------
		#region Filter

		// ---------------------------------------------------------------------
		private void InicializovatFiltre(int? vynutitFilterID = null)
		{
			App.Uzamknut();
			try // finally
			{
				int? povodnyFilterID = null;
				if (ListBoxFilter.SelectedValue != null)
					povodnyFilterID = ((MesacnaStatistikaFilter)ListBoxFilter.SelectedValue).FilterID;

				// Odfilturjeme data
				var data = DB.Layer.NacitatFiltre();

				// Refresneme zoznam
				ListBoxFilter.ItemsSource = data.OrderBy(item => item.Popis);

				// Pokusime sa oznacit vynuteny
				if (vynutitFilterID.HasValue)
					ListBoxFilter.Oznacit<MesacnaStatistikaFilter>(item => item.FilterID == vynutitFilterID.Value);
				
				// Ak neexistuje, skusime oznacit predchadzajuci
				if (povodnyFilterID.HasValue && ListBoxFilter.SelectedValue == null)
					ListBoxFilter.Oznacit<MesacnaStatistikaFilter>(item => item.FilterID == povodnyFilterID.Value);

				InicializovatPodmienkyFiltra();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		// ---------------------------------------------------------------------
		private void IgnorovatFilter()
		{
			if (ListBoxFilter.SelectedValue == null)
				return;
				
			App.Uzamknut();
			try // finally
			{
				// Updatujeme polozku
				MesacnaStatistikaFilter oznacenaPolozka = (MesacnaStatistikaFilter)ListBoxFilter.SelectedValue;
				DB.Layer.FilterIgnorovat(oznacenaPolozka.FilterID.Value, !oznacenaPolozka.Ignorovat);

				// Obnovime zoznam
				InicializovatFiltre();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		// ---------------------------------------------------------------------
		private void VymazatFilter()
		{
			if (ListBoxFilter.SelectedValue == null)
				return;

			App.Uzamknut();
			try // finally
			{
				// Updatujeme polozku
				DB.Layer.FilterVymazat(((MesacnaStatistikaFilter)ListBoxFilter.SelectedValue).FilterID.Value);

				// Obnovime zoznam
				InicializovatFiltre();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		// ---------------------------------------------------------------------
		private void EditovatFilter()
		{
			if (ListBoxFilter.SelectedValue == null)
				return;

			App.Uzamknut();
			try // finally
			{
				// Updatujeme polozku
				FilterDialog.EditovatZaznam(((MesacnaStatistikaFilter)ListBoxFilter.SelectedValue).FilterID.Value);

				// Obnovime zoznam
				InicializovatFiltre();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		#endregion // Filter

		// ---------------------------------------------------------------------
		#region Podmienka Filtra

		// ---------------------------------------------------------------------
		private void InicializovatPodmienkyFiltra(int? vynutitFilterPodmienkaID = null)
		{
			ListBoxFilterPodmienka.ItemsSource = null;
			if (ListBoxFilter.SelectedValue == null)
				return;

			App.Uzamknut();
			try // finally
			{
				int? povodnyFilterPodmienkaID = null;
				if (ListBoxFilterPodmienka.SelectedValue != null)
					povodnyFilterPodmienkaID = ((MesacnaStatistikaFilterPodmienka)ListBoxFilterPodmienka.SelectedValue).FilterPodmienkaID;

				// Odfilturjeme data
				var data = DB.Layer.NacitatPodmienkyFiltra(((MesacnaStatistikaFilter)ListBoxFilter.SelectedValue).FilterID.Value);

				// Refresneme zoznam
				ListBoxFilterPodmienka.ItemsSource = data.OrderBy(item => item.Format);

				// Pokusime sa oznacit vynuteny
				if (vynutitFilterPodmienkaID.HasValue)
					ListBoxFilterPodmienka.Oznacit<MesacnaStatistikaFilterPodmienka>(item => item.FilterPodmienkaID == vynutitFilterPodmienkaID.Value);
				
				// Ak neexistuje, skusime oznacit predchadzajuci
				if (povodnyFilterPodmienkaID.HasValue && ListBoxFilterPodmienka.SelectedValue == null)
					ListBoxFilterPodmienka.Oznacit<MesacnaStatistikaFilterPodmienka>(item => item.FilterPodmienkaID == povodnyFilterPodmienkaID.Value);
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		// ---------------------------------------------------------------------
		private void VymazatFilterPodmienka()
		{
			if (ListBoxFilterPodmienka.SelectedValue == null)
				return;

			App.Uzamknut();
			try // finally
			{
				// Updatujeme polozku
				DB.Layer.FilterPodmienkaVymazat(((MesacnaStatistikaFilterPodmienka)ListBoxFilterPodmienka.SelectedValue).FilterPodmienkaID.Value);

				// Obnovime zoznam
				InicializovatPodmienkyFiltra();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		// ---------------------------------------------------------------------
		private void EditovatFilterPodmienka()
		{
			if (ListBoxFilterPodmienka.SelectedValue == null)
				return;

			App.Uzamknut();
			try // finally
			{
				// Updatujeme polozku
				FilterPodmienkaDialog.EditovatZaznam(
					(ListBoxFilterPodmienka.SelectedValue as MesacnaStatistikaFilterPodmienka),
					(ListBoxFilter.SelectedValue as MesacnaStatistikaFilter).FilterID.Value);

				// Obnovime zoznam
				InicializovatFiltre();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		#endregion // Podmienka
	} // public partial class StatistikaMesiacPrisposobit

} // namespace EvidenciaUctov.Dialogy