﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov.Dialogy
{
	// *************************************************************************
	// *** StatistikaZakladDialog **********************************************
	// *************************************************************************
	public partial class StatistikaZakladDialog : Window
	{
		// ---------------------------------------------------------------------
		private const int ZarovnanieText = 20;
		public const int ZarovnanieCiastka = 15;
		// ---------------------------------------------------------------------
		private Obrat[] pohyby;

		// ---------------------------------------------------------------------
		// --- StatistikaZakladDialog Konstruktor ------------------------------
		// ---------------------------------------------------------------------
		public StatistikaZakladDialog(Obrat[] pohyby)
		{
			InitializeComponent();

			this.Inicializovat();
			this.pohyby = pohyby;
		}

		// ---------------------------------------------------------------------
		// --- StatistikaZakladDialog Udalosti ---------------------------------
		// ---------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (pohyby == null)
				return;

			App.Uzamknut();
			try // finally
			{
				TextBoxLog.Clear();

				// Zobrazime chyby
				TextBoxLog.Text += "Počet pohybov: " + pohyby.Count().ToString() + Environment.NewLine;
				if (pohyby.Count() > 0)
					TextBoxLog.Text += ZobrazitVysledky(pohyby);
			} // try
			finally
			{
				App.Odomknut();
			} // finally
		}

		// ---------------------------------------------------------------------
		// --- StatistikaZakladDialog Metody -----------------------------------
		// ---------------------------------------------------------------------
		private static string ZobrazitVysledky(Obrat[] pohyby)
		{
			if (pohyby == null || pohyby.Count() == 0)
				return string.Empty;

			// Zoskupime podla druhu obratu
			var zoznam =
				from pohyb in pohyby
				where
					!pohyb.Ignorovat &&
					pohyb.DruhObratu != DruhObratu.Irelevantne && pohyb.DruhObratu != DruhObratu.Neznamy
				group pohyb by pohyb.DruhObratu into groupBy
				select new { DruhObratu = groupBy.Key, SumaCistky = groupBy.Sum(suma => suma.Ciastka) };

			// Vypiseme zoznam
			List<string> vysledok = new List<string>();
			foreach (var pohyb in zoznam)
				vysledok.Add((pohyb.DruhObratu.Description() + ":").PadRight(ZarovnanieText) + pohyb.SumaCistky.ToString("C2").PadLeft(ZarovnanieCiastka));

			// Agregovane vydavky
			var zoznamAgregovany =
				from pohyb in zoznam
				where
					pohyb.DruhObratu != DruhObratu.Vklad && pohyb.DruhObratu != DruhObratu.Urok
				select pohyb.SumaCistky;

			vysledok.Add(string.Empty);
			vysledok.Add("Výdavky spolu:".PadRight(ZarovnanieText) + zoznamAgregovany.Sum().ToString("C2").PadLeft(ZarovnanieCiastka));

			return string.Join(Environment.NewLine, vysledok.ToArray());
		}
	} // public partial class StatistikaZakladDialog

} // namespace EvidenciaUctov.Dialogy