﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using EvidenciaUctov.Utils;

namespace EvidenciaUctov
{
	// *************************************************************************
	// *** MainWindow **********************************************************
	// *************************************************************************
	public partial class MainWindow : Window
	{
		// ---------------------------------------------------------------------
		private Filter filter;

		// ---------------------------------------------------------------------
		// --- MainWindow Konstruktor ------------------------------------------
		// ---------------------------------------------------------------------
		public MainWindow()
		{
			InitializeComponent();

			Title += " " + App.Verzia;
		}

		// ---------------------------------------------------------------------
		// --- MainWindow Vlastnosti -------------------------------------------
		// ---------------------------------------------------------------------
		public Ucet VybranyUcet
		{
			get { return ComboBoxUcet.SelectedValue as Ucet; } // get
		}

		// ---------------------------------------------------------------------
		// --- MainWindow Udalost ----------------------------------------------
		// ---------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (!Prihlasit())
				Close();
		}

		// ---------------------------------------------------------------------
		private void ComboBoxUcet_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
		{
			if (VybranyUcet != null)
				InicializovatObraty();
		}

		// ---------------------------------------------------------------------
		private void ListBoxObraty_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.F8)
			{
				Dispatcher.Focus(ListBoxObraty);
				IgnorovatOznaceny();
			} // if
		}

		// ---------------------------------------------------------------------
		private void Window_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.Key)
			{
				case Key.F5:
				{
					if (VybranyUcet != null)
						InicializovatObraty();
					break;
				} // F5

				case Key.Delete:
				{
					VymazatOznaceny();
					break;
				} // Del

				case Key.F:
				{
					if (Keyboard.IsKeyDown(Key.LeftCtrl))
						Vyhladat();

					break;
				} // F

				case Key.F3:
				{
					if (!string.IsNullOrEmpty(Properties.Settings.Default.PosledneHladanie))
						Vyhladat(Properties.Settings.Default.PosledneHladanie);

					break;
				} // F3
			} // if
		}

		// ---------------------------------------------------------------------
		private void ButtonNacitat_Click(object sender, RoutedEventArgs e)
		{
			if (VybranyUcet == null)
				throw new Exception("Musí existovať nejaký bankový účet!");

			var dialog = new Microsoft.Win32.OpenFileDialog();
			if (dialog.ShowDialog() ?? false)
			{
				App.Uzamknut();
				try // finally
				{
					// Nacitame subor
					string[] log;
					var pohyby = VybranyUcet.Banka.VytvoritParserPohybu().Parsuj(dialog.FileName, out log);
					if (log != null && log.Length > 0)
						new Dialogy.PoznamkyDialog(log).ShowDialog();

					// Ulozime vysledky (ak nejake su)
					if (pohyby != null && pohyby.Count() > 0)
					{
						ProgressInit(pohyby.Count());
						try // finally
						{
							foreach (Obrat obrat in pohyby)
							{
								ProgressBar.Value += 1;
								WpfUtils.DoEvents();	// Prekreslime GUI

								DB.Layer.ObratUlozit(obrat, VybranyUcet.UcetID.Value);
							} // foreach
						} // try
						finally
						{
							ProgressInit();
						} // finally

						// Zobrazime spravu a obnovime zoznam
						App.Sprava("Úspešne naimportované záznamy.");
						if (VybranyUcet != null)
							InicializovatObraty();
					} // if
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if	
		}

		// ---------------------------------------------------------------------
		private void ToolbarMainControl_ClickHladat(object sender, RoutedEventArgs e)
		{
			Vyhladat();
		}

		// ---------------------------------------------------------------------
		private void ButtonNovyUcet_Click(object sender, RoutedEventArgs e)
		{
			NovyUcet();
		}

		// ---------------------------------------------------------------------
		private void ButtonIgnorovat_Click(object sender, RoutedEventArgs e)
		{
			IgnorovatOznaceny();
		}

		// ---------------------------------------------------------------------
		private void ButtonVymazat_Click(object sender, RoutedEventArgs e)
		{
			VymazatOznaceny();
		}

		// ---------------------------------------------------------------------
		private void ButtonObnovit_Click(object sender, RoutedEventArgs e)
		{
			if (VybranyUcet != null)
				InicializovatObraty();
		}

		// ---------------------------------------------------------------------
		#region Menu

		// ---------------------------------------------------------------------
		private void MenuItemSuborNovyUcet_Click(object sender, RoutedEventArgs e)
		{
			NovyUcet();
		}

		// ---------------------------------------------------------------------
		private void MenuItemSuborUkoncit_Click(object sender, RoutedEventArgs e)
		{
			Close();
		}

		// ---------------------------------------------------------------------
		private void MenuItemStatistikyZoSuboru_Click(object sender, RoutedEventArgs e)
		{
			// Zvolime druh parseru
			Banka? pouzitBanku = null;
			if (EnumUtils.GetEnumList<Banka>().Count() == 2)
				pouzitBanku = Banka.Slsp;	// Ak je jedina banka, vynutime tuto
			else
			{
				var dialogBanka = new Dialogy.ZvolitBankuDialog();
				if (dialogBanka.ShowDialog() ?? false)
					pouzitBanku = dialogBanka.VybranaBanka.Value;
			} // if

			var dialog = new Microsoft.Win32.OpenFileDialog();
			if (pouzitBanku.HasValue && (dialog.ShowDialog() ?? false))
			{
				App.Uzamknut();
				try // finally
				{
					// Nacitame subor
					string[] log;
					var pohyby = pouzitBanku.Value.VytvoritParserPohybu().Parsuj(dialog.FileName, out log);
					if (log != null && log.Length > 0)
						new Dialogy.PoznamkyDialog(log).ShowDialog();

					// Zobrazime vysledky
					var dialogStatistika = new Dialogy.StatistikaZakladDialog(pohyby);
					dialogStatistika.ShowDialog();
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if
		}

		// ---------------------------------------------------------------------
		private void MenuItemStatistikyZoZoznamu_Click(object sender, RoutedEventArgs e)
		{
			if (ListBoxObraty.Items.Count > 0)
				new Dialogy.StatistikaZakladDialog(ListBoxObraty.ItemsSource.Cast<Obrat>().ToArray()).ShowDialog();
		}

		// ---------------------------------------------------------------------
		private void MenuItemStatistikyMesacna_Click(object sender, RoutedEventArgs e)
		{
			if (VybranyUcet != null)
				new Dialogy.StatistikaMesiac(VybranyUcet.UcetID.Value).ShowDialog();
		}

		// ---------------------------------------------------------------------
		private void MenuItemNastrojePrihlasit_Click(object sender, RoutedEventArgs e)
		{
			Prihlasit();
		}

		// ---------------------------------------------------------------------
		private void MenuItemNastrojeSkupinaFiltrov_Click(object sender, RoutedEventArgs e)
		{
			new Dialogy.SkupinyFitrovDialog().ShowDialog();
		}

		// ---------------------------------------------------------------------
		private void MenuItemNastrojeFiltre_Click(object sender, RoutedEventArgs e)
		{
			new Dialogy.StatistikaMesiacPrisposobit().ShowDialog();
		}

		// ---------------------------------------------------------------------
		private void MenuItemFilterVybrat_Click(object sender, RoutedEventArgs e)
		{
			var dialog = new Dialogy.FilterVybrat();
			if (dialog.ShowDialog() ?? false)
			{
				filter = XmlUtils.Load<Filter>(dialog.OznaceneData.Data);

				InicializovatObraty();
				ObnovitStavFiltra();
			} // if
		}

		// ---------------------------------------------------------------------
		private void MenuItemFilterPridat_Click(object sender, RoutedEventArgs e)
		{
			var dialog = new Dialogy.FilterPridat();
			if (dialog.ShowDialog() ?? false)
			{
				if (filter == null)
					filter = new Filter(dialog.Filter);
				else
				{
					var operatorDialog = new Dialogy.FilterLogickyOperator();
					if (operatorDialog.ShowDialog() ?? false)
						filter.Pridat(dialog.Filter, operatorDialog.Operator);
				} // else

				InicializovatObraty();
				ObnovitStavFiltra();
			} // if
		}

		// ---------------------------------------------------------------------
		private void MenuItemFilterUlozit_Click(object sender, RoutedEventArgs e)
		{
			var dialog = new Dialogy.FilterUlozit();
			if (dialog.ShowDialog() ?? false)
			{
				App.Uzamknut();
				try // finally
				{
					var xml = XmlUtils.Save(filter);
					DB.Layer.VlozitData(DB.Layer.TypDat.SerializovanyFilter, dialog.Nazov, xml.ToString());
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if
		}

		// ---------------------------------------------------------------------
		private void MenuItemFilterZrusit_Click(object sender, RoutedEventArgs e)
		{
			filter = null;

			InicializovatObraty();
			ObnovitStavFiltra();
		}

		// ---------------------------------------------------------------------
		private void MenuItemPomocOPrograme_Click(object sender, RoutedEventArgs e)
		{
			new Dialogy.OProgrameDialog().ShowDialog();
		}

		#endregion // Menu

		// ---------------------------------------------------------------------
		// --- MainWindow Metody -----------------------------------------------
		// ---------------------------------------------------------------------
		private void Inicializovat()
		{
			ListBoxObraty.ItemsSource = null;
			InicializovatUcty();

			// Ak neexistuje ziaden ucet, skusime vynutit u uzivatela vytvorenie noveho
			if (ComboBoxUcet.Items.Count == 0)
				NovyUcet();

			ObnovitStavFiltra();
		}

		// ---------------------------------------------------------------------
		private void InicializovatUcty(int? oznacitUcetID = null)
		{
			App.Uzamknut();
			try // finally
			{
				// Zapamatame si predchadzajuci oznaceny ucet
				int? predchadzajuciOznacenyUcetID = null;
				if (ComboBoxUcet.SelectedValue != null)
					predchadzajuciOznacenyUcetID = ((Ucet)ComboBoxUcet.SelectedValue).UcetID;

				// Updatneme zoznam
				ComboBoxUcet.ItemsSource = DB.Layer.NacitatUcty().OrderBy(item => item.Nazov);
				
				// Vyberieme nejaky ucet
				if (oznacitUcetID.HasValue)
				{
					// Oznacime ten, ktory bol vynuteny
					ComboBoxUcet.Oznacit<Ucet>(item => item.UcetID == oznacitUcetID);
				} // if
				else
				{
					// Ak sme nevynutili konkretny (napr. ked sa vytvoril novy), tak skusime oznacit predchadzajuci
					if (predchadzajuciOznacenyUcetID.HasValue)
						ComboBoxUcet.Oznacit<Ucet>(item => item.UcetID == predchadzajuciOznacenyUcetID);

					// Ak sa nepodarilo oznacit predchadzajuci - skusime oznacit prvy ktory najdeme
					if (ComboBoxUcet.SelectedValue == null && ComboBoxUcet.Items.Count > 0)
						ComboBoxUcet.SelectedIndex = 0;
				} // else

				// Neupdatuje sa cez event
				if (VybranyUcet != null)
					InicializovatObraty();
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}

		// ---------------------------------------------------------------------
		private void NovyUcet()
		{
			int? ucetID;
			if (Dialogy.UcetDialog.NovyUcet(out ucetID))
				InicializovatUcty(ucetID);
		}

		// ---------------------------------------------------------------------
		private void ProgressInit(int max = 0)
		{
			ProgressBar.Value = 0;
			ProgressBar.Maximum = max;
		}

		// ---------------------------------------------------------------------
		private void InicializovatObraty()
		{
			InicializovatObraty(VybranyUcet.UcetID, FilterControlMain.DajFilter(), filter);
		}

		// ---------------------------------------------------------------------
		private void InicializovatObraty(int? ucetID, FilterObdobie filterObdobie, Filter filter)
		{
			TextBlockPocet.Text = string.Empty;

			App.Uzamknut();
			try // finally
			{
				if (ucetID.HasValue)
				{
					int? oznacenyObratID = null;
					if (ListBoxObraty.SelectedValue != null)
						oznacenyObratID = ((Obrat)ListBoxObraty.SelectedValue).ObratID;

					// Odfilturjeme data
					var data = DB.Layer.NacitatObraty(ucetID.Value, filterObdobie, filter);
					
					// Refresneme zoznam
					ListBoxObraty.ItemsSource = data.OrderBy(item => item.Datum);
					TextBlockPocet.Text = "Počet: " + data.Count().ToString();

					// Pokusime obnovit predchadzajuci kurtoz (ak existuje)
					if (oznacenyObratID.HasValue)
						ListBoxObraty.Oznacit<Obrat>(item => item.ObratID == oznacenyObratID.Value);
				} // if
			} // try
			finally
			{
				App.Odomknut();
			} // finallt
		}
	
		// ---------------------------------------------------------------------
		private bool Prihlasit()
		{
			var dialog = new Dialogy.PrihlasovaciDialog();
			if (dialog.ShowDialog() ?? false)
			{
				Title += " " + DB.Layer.Databaza;
				Inicializovat();
				return true;
			} // if

			return false;
		}

		// ---------------------------------------------------------------------
		private void IgnorovatOznaceny()
		{
			if (ListBoxObraty.SelectedValue != null)
			{
				App.Uzamknut();
				try // finally
				{
					var oznacenyObrat = (Obrat)ListBoxObraty.SelectedValue;
					DB.Layer.ObratIgnorovat(oznacenyObrat.ObratID.Value, !oznacenyObrat.Ignorovat);

					InicializovatObraty();
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if
		}

		// ---------------------------------------------------------------------
		private void VymazatOznaceny()
		{
			if (ListBoxObraty.SelectedValue != null)
			{
				App.Uzamknut();
				try // finally
				{
					var oznacenyObrat = (Obrat)ListBoxObraty.SelectedValue;
					DB.Layer.ObratVymazat(oznacenyObrat.ObratID.Value);

					InicializovatObraty();
				} // try
				finally
				{
					App.Odomknut();
				} // finally
			} // if
		}

		// ---------------------------------------------------------------------
		private void Vyhladat()
		{
			var dialog = new Dialogy.HladatDialog();
			if (dialog.ShowDialog() ?? false)
				Vyhladat(dialog.Text);
		}

		// ---------------------------------------------------------------------
		private void Vyhladat(string text)
		{
			// Vyhladame vsetky zaznamy, ktore vyhovuju
			var zoznamVyhovujucichID = 
				from item in ListBoxObraty.ItemsSource.Cast<Obrat>()
				where item.ToString().ToLower().Contains(text.ToLower())
				select item.ObratID;

			if (zoznamVyhovujucichID.Count() == 0)
				throw new Exception("Záznam '" + text + "' neexistuje.");

			// Ak je oznaceny nejaky zaznam, skusime hladat pod nim
			var oznacenyZaznam = (ListBoxObraty.SelectedValue as Obrat?);
			if (oznacenyZaznam.HasValue)
			{
				bool smeZaOznacenym = false;
				foreach (Obrat polozka in ListBoxObraty.Items)
				{
					if (smeZaOznacenym)
					{
						if (zoznamVyhovujucichID.Contains(polozka.ObratID))
						{
							ListBoxObraty.Oznacit<Obrat>(item => item.ObratID == polozka.ObratID);
							return;
						} // if
					} // if

					if (polozka.Identifikator == oznacenyZaznam.Value.Identifikator)
						smeZaOznacenym = true;
				} // foreach
			} // if

			// Ak nenasiel ziaden pod oznacenout, resp. nie je nic oznacene - zoberieme prvy zaznam v liste
			ListBoxObraty.Oznacit<Obrat>(item => item.ObratID == zoznamVyhovujucichID.First());
		}

		// ---------------------------------------------------------------------
		private void ObnovitStavFiltra()
		{
			TextBlockWarning.Text = (filter != null) ? "Zoznam je filtrovaný!" : string.Empty;
			MenuItemFilterUlozit.IsEnabled = MenuItemFilterZrusit.IsEnabled = (filter != null);
		}
	} // public partial class MainWindow

} // namespace EvidenciaUctov