﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Controls.Primitives;

namespace EvidenciaUctov.Utils
{
	// *************************************************************************
	// *** EnumUtils ***********************************************************
	// *************************************************************************
	public static class EnumUtils
	{
		// ---------------------------------------------------------------------
		public static T GetFirstAttribute<T>(Enum value) where T : class
		{
			if (value != null)
			{
				var attribute = (T[])(value.GetType().GetField(value.ToString()).GetCustomAttributes(typeof(T), /*inherited*/false));

				if (attribute != null && attribute.Length > 0)
					return attribute[0];
			} // if

			return null;
		}

		// ---------------------------------------------------------------------
		public static string Description(this Enum enumValue)
		{
			var attribute = GetFirstAttribute<DescriptionAttribute>(enumValue);
			return attribute == null ? enumValue.ToString() : attribute.Description;
		}

		// ---------------------------------------------------------------------
		public static IEnumerable<T> GetEnumList<T>(params T[] vynechat) where T : struct
		{
			return new List<T>(Enum.GetValues(typeof(T)).Cast<T>()).Where(item => !vynechat.Contains(item)).AsEnumerable();
		}

		// ---------------------------------------------------------------------
		public static IEnumerable<KeyValuePair<string, T>> GetEnumDictionaryList<T>(params T[] vynechat) where T : struct
		{
			return from item in GetEnumList(vynechat) select new KeyValuePair<string, T>(Description(item as Enum), item);
		}

		// ---------------------------------------------------------------------
		public static IEnumerable<KeyValuePair<string, T>> GetEnumDictionaryListFromArray<T>(T[] zoznam) where T : struct
		{
			return from item in zoznam select new KeyValuePair<string, T>(Description(item as Enum), item);
		}

		// ---------------------------------------------------------------------
		public static void SetEnumDictionaryListValue<T>(this Selector control, T value) where T : struct
		{
			var list = (IEnumerable<KeyValuePair<string, T>>)control.ItemsSource;

			control.SelectedValue = (from item in list where item.Value.Equals(value) select item).FirstOrDefault();
		}

		// ---------------------------------------------------------------------
		public static T GetEnumValue<T>(this object value) where T : struct
		{
			return ((KeyValuePair<string, T>)value).Value;
		}
	} // public static class EnumUtils

} // namespace EvidenciaUctov.Utils