﻿using System.Collections.Generic;
using System.Xml.Serialization;

namespace EvidenciaUctov.Utils
{
	// *************************************************************************
	// *** SerializableDictionary **********************************************
	// *************************************************************************
	/// <summary>
	/// http://weblogs.asp.net/pwelter34/archive/2006/05/03/444961.aspx
	/// </summary>
	/// <typeparam name="TKey"></typeparam>
	/// <typeparam name="TValue"></typeparam>
	[XmlRoot("dictionary")]
	public class SerializableDictionary<TKey, TValue> : Dictionary<TKey, TValue>, IXmlSerializable
	{
		// ---------------------------------------------------------------------
		public System.Xml.Schema.XmlSchema GetSchema()
		{
			return null;
		}

		// ---------------------------------------------------------------------
		public void ReadXml(System.Xml.XmlReader reader)
		{
			var keySerializer = new XmlSerializer(typeof(TKey));
			var valueSerializer = new XmlSerializer(typeof(TValue));
			bool wasEmpty = reader.IsEmptyElement;

			reader.Read();
			if (wasEmpty)
				return;

			while (reader.NodeType != System.Xml.XmlNodeType.EndElement)
			{
				reader.ReadStartElement("item");
				reader.ReadStartElement("key");
				var key = (TKey)keySerializer.Deserialize(reader);
				reader.ReadEndElement();
				reader.ReadStartElement("value");
				var value = (TValue)valueSerializer.Deserialize(reader);
				reader.ReadEndElement();

				Add(key, value);

				reader.ReadEndElement();
				reader.MoveToContent();
			}

			reader.ReadEndElement();
		}

		// ---------------------------------------------------------------------
		public void WriteXml(System.Xml.XmlWriter writer)
		{
			var keySerializer = new XmlSerializer(typeof(TKey));
			var valueSerializer = new XmlSerializer(typeof(TValue));

			foreach (TKey key in this.Keys)
			{
				writer.WriteStartElement("item");

				writer.WriteStartElement("key");
				keySerializer.Serialize(writer, key);
				writer.WriteEndElement();

				writer.WriteStartElement("value");
				TValue value = this[key];
				valueSerializer.Serialize(writer, value);

				writer.WriteEndElement();
				writer.WriteEndElement();
			}
		}
	} // public class SerializableDictionary<TKey, TValue>

} // namespace EvidenciaUctov.Utils