﻿using System;
using System.Collections.Generic;
using System.Globalization;

namespace EvidenciaUctov.Utils
{
	// *************************************************************************
	// *** StringUtils *********************************************************
	// *************************************************************************
	public static class StringUtils
	{
		// ---------------------------------------------------------------------
		// --- StringUtils Methods ---------------------------------------------
		// ---------------------------------------------------------------------
		public static bool IsInt(this string value)
		{
			return value.ToInt() != null;
		}

		// ---------------------------------------------------------------------
		public static int? ToInt(this string value, int? defaultValue = null)
		{
			int result;
			return int.TryParse(value, NumberStyles.Float, Global.DefaultCulture, out result) ? result : defaultValue;
		}

		// ---------------------------------------------------------------------
		public static double? ToDouble(this string value, double? defaultValue = null)
		{
			double result;
			if (double.TryParse(value, NumberStyles.Float, Global.DefaultCulture, out result))
				return result;
	
			// Pokusime sa zmenit oddelovac desatinych miest
			if (!string.IsNullOrEmpty(value))
				value = value.Replace(".", Global.DefaultCulture.NumberFormat.CurrencyDecimalSeparator);

			return double.TryParse(value, NumberStyles.Float, Global.DefaultCulture, out result) ? result : defaultValue;
		}

		// --------------------------------------------------------------------- 
		public static bool IsDateTime(this string value)
		{
			return (value.ToDateTime() != null);
		}

		// --------------------------------------------------------------------- 
		public static DateTime? ToDateTime(this string value, DateTime? defaultValue = null)
		{
			DateTime result;
			return DateTime.TryParse(value, Global.DefaultCulture.DateTimeFormat, DateTimeStyles.AdjustToUniversal, out result) ? result : defaultValue;
		}

		// --------------------------------------------------------------------- 
		public static string ToDelimitedString(this List<string> texts, string delimiter = ", ", bool removeEmpty = true)
		{
			if (removeEmpty)
				texts.RemoveAll(item => string.IsNullOrEmpty(item));
			
			return string.Join(delimiter, texts.ToArray());
		}
	} // public static class StringUtils

} // namespace EvidenciaUctov.Utils