﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using System.Windows.Controls.Primitives;

namespace EvidenciaUctov.Utils
{
	// *************************************************************************
	// *** WpfUtils ************************************************************
	// *************************************************************************
	public static class WpfUtils
	{
		// ---------------------------------------------------------------------
		// --- WpfUtils Metody -------------------------------------------------
		// ---------------------------------------------------------------------
		public static void Focus(this Dispatcher dispatcher, UIElement element)
		{
			dispatcher.BeginInvoke(DispatcherPriority.Background, new Action(delegate() { element.Focus(); }));
		}

		// ---------------------------------------------------------------------
		public static void Oznacit<T>(this Selector control, Func<T, bool> predicate)
		{
			var item = control.ItemsSource.OfType<T>().FirstOrDefault(predicate);

			var listBox = (control as ListBox);
			if (item != null && listBox != null)
			{

				// Tato uchylarna je kvoli tomu, aby sme pri zmeneni fokusu nastavili aj klavesnicu na danu polozku (nie len GUI) -
				// teda aby sipka dole isla od fokusovanej polozky
				EventHandler smernikNaEvent = null;
				smernikNaEvent = (object sender, EventArgs e) =>
					{
						if (listBox.ItemContainerGenerator.Status == System.Windows.Controls.Primitives.GeneratorStatus.ContainersGenerated)
						{
							int selectedIndex = (control as ListBox).SelectedIndex;
							if (selectedIndex < 0)
							{
								// Uvolnime event, aj tak nie je nic oznacene
								control.ItemContainerGenerator.StatusChanged -= smernikNaEvent;
							} // else
							else
							{
								ListBoxItem listBoxItem = listBox.ItemContainerGenerator.ContainerFromIndex(selectedIndex) as ListBoxItem;
								if (listBoxItem != null)
								{
									Dispatcher.CurrentDispatcher.Focus(listBoxItem);
									control.ItemContainerGenerator.StatusChanged += null;

									// Uvolnime event, aby sme nepridavali eventy pri kazdom jednom pouziti
									control.ItemContainerGenerator.StatusChanged -= smernikNaEvent;
								} // if
							} // else
						} // if
					};

				control.ItemContainerGenerator.StatusChanged += smernikNaEvent;
			} // if

			control.SelectedValue = control.ItemsSource.OfType<T>().FirstOrDefault(predicate);
			
			// Zmenime scroll bar podla oznacenej pozicie
			if (listBox != null)
				listBox.ScrollIntoView(listBox.SelectedItem);
		}

		// ---------------------------------------------------------------------
		public static void DoEvents()
		{
			DispatcherFrame frame = new DispatcherFrame(/*exitWheNRequest*/true);
			Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.Background,
				(System.Threading.SendOrPostCallback) delegate (object arg)
				{
					var f = arg as DispatcherFrame;
					f.Continue = false;
				},
				frame);
			Dispatcher.PushFrame(frame);
		}

		// ---------------------------------------------------------------------
		public static Visibility ToVisibility(this bool value)
		{
			return value ? Visibility.Visible : Visibility.Collapsed;
		}

		// ---------------------------------------------------------------------
		public static void ButtonIsEnabledChanged(this Button button)
		{
			Image image = button.Content as Image;
			if (image != null)
				image.Opacity = button.IsEnabled ? 1.0 : 0.2;
		}
	} // public static class WpfUtils

} // namespace EvidenciaUctov.Utils